@@ -54,6 +54,7 @@ class MediaWikiPage(object):
         "_revision_id",
         "_parent_id",
         "_html",
+        "_soup",
         "_images",
         "_references",
         "_categories",
@@ -106,6 +107,7 @@ class MediaWikiPage(object):
         self._table_of_contents = None
         self._logos = None
         self._hatnotes = None
+        self._soup = None
 
         self.__load(redirect=redirect, preload=preload)
 
@@ -262,8 +264,10 @@ class MediaWikiPage(object):
                 This is a parsing operation and not part of the standard API"""
         if self._logos is None:
             self._logos = list()
-            soup = BeautifulSoup(self.html, "html.parser")
-            info = soup.find("table", {"class": "infobox"})
+            # Cache the results of parsing the html, so that multiple calls happen much faster
+            if not self._soup:
+                self._soup = BeautifulSoup(self.html, "html.parser")
+            info = self._soup.find("table", {"class": "infobox"})
             if info is not None:
                 children = info.find_all("a", class_="image")
                 for child in children:
@@ -282,8 +286,10 @@ class MediaWikiPage(object):
                 This is a parsing operation and not part of the standard API"""
         if self._hatnotes is None:
             self._hatnotes = list()
-            soup = BeautifulSoup(self.html, "html.parser")
-            notes = soup.find_all("div", class_="hatnote")
+            # Cache the results of parsing the html, so that multiple calls happen much faster
+            if not self._soup:
+                self._soup = BeautifulSoup(self.html, "html.parser")
+            notes = self._soup.find_all("div", class_="hatnote")
             if notes is not None:
                 for note in notes:
                     tmp = list()
@@ -466,9 +472,12 @@ class MediaWikiPage(object):
         """ Plain text section content
 
             Args:
-                section_title (str): Name of the section to pull
+                section_title (str): Name of the section to pull or None \
+                    for the header section
             Returns:
                 str: The content of the section
+            Note:
+                Use **None** if the header section is desired
             Note:
                 Returns **None** if section title is not found; only text \
                 between title and next section or sub-section title is returned
@@ -476,21 +485,30 @@ class MediaWikiPage(object):
                 Side effect is to also pull the content which can be slow
             Note:
                 This is a parsing operation and not part of the standard API"""
-        section = "== {0} ==".format(section_title)
-        try:
-            content = self.content
-            index = content.index(section) + len(section)
+        if not section_title:
+            try:
+                content = self.content
+                index = 0
+            except ValueError:
+                return None
+            except IndexError:
+                pass
+        else:
+            section = "== {0} ==".format(section_title)
+            try:
+                content = self.content
+                index = content.index(section) + len(section)
 
-            # ensure we have the full section header...
-            while True:
-                if content[index + 1] == "=":
-                    index += 1
-                else:
-                    break
-        except ValueError:
-            return None
-        except IndexError:
-            pass
+                # ensure we have the full section header...
+                while True:
+                    if content[index + 1] == "=":
+                        index += 1
+                    else:
+                        break
+            except ValueError:
+                return None
+            except IndexError:
+                pass
 
         try:
             next_index = self.content.index("==", index)
@@ -503,17 +521,27 @@ class MediaWikiPage(object):
         """ Parse all links within a section
 
             Args:
-                section_title (str): Name of the section to pull
+                section_title (str): Name of the section to pull or, if \
+                    None is provided, the links between the main heading and \
+                    the first section
             Returns:
                 list: List of (title, url) tuples
+            Note:
+                Use **None** to pull the links from the header section
             Note:
                 Returns **None** if section title is not found
             Note:
                 Side effect is to also pull the html which can be slow
             Note:
                 This is a parsing operation and not part of the standard API"""
-        soup = BeautifulSoup(self.html, "html.parser")
-        headlines = soup.find_all("span", class_="mw-headline")
+        # Cache the results of parsing the html, so that multiple calls happen much faster
+        if not self._soup:
+            self._soup = BeautifulSoup(self.html, "html.parser")
+
+        if not section_title:
+            return self._parse_section_links(None)
+
+        headlines = self._soup.find_all("span", class_="mw-headline")
         tmp_soup = BeautifulSoup(section_title, "html.parser")
         tmp_sec_title = tmp_soup.get_text().lower()
         id_tag = None
@@ -664,14 +692,20 @@ class MediaWikiPage(object):
 
     def _parse_section_links(self, id_tag):
         """ given a section id, parse the links in the unordered list """
-        soup = BeautifulSoup(self.html, "html.parser")
-        info = soup.find("span", {"id": id_tag})
         all_links = list()
 
-        if info is None:
-            return all_links
+        if id_tag is None:
+            root = self._soup.find("div", {"class": "mw-parser-output"})
+            if root is None:
+                return all_links
+            candidates = root.children
+        else:
+            root = self._soup.find("span", {"id": id_tag})
+            if root is None:
+                return all_links
+            candidates = self._soup.find(id=id_tag).parent.next_siblings
 
-        for node in soup.find(id=id_tag).parent.next_siblings:
+        for node in candidates:
             if not isinstance(node, Tag):
                 continue
             if node.get("role", "") == "navigation":
@@ -679,6 +713,13 @@ class MediaWikiPage(object):
             elif "infobox" in node.get("class", []):
                 continue
 
+            # If the classname contains "toc", the element is a table of contents.
+            # The comprehension is necessary because there are several possible
+            # types of tocs: "toclevel", "toc", ...
+            toc_classnames = [cname for cname in node.get("class", []) if "toc" in cname]
+            if toc_classnames:
+                continue
+
             # this is actually the child node's class...
             is_headline = node.find("span", {"class": "mw-headline"})
             if is_headline is not None:
@@ -834,7 +875,7 @@ class MediaWikiPage(object):
             """ parse the category correctly """
             tmp = val["title"]
             if tmp.startswith(self.mediawiki.category_prefix):
-                return tmp[len(self.mediawiki.category_prefix) + 1 :]
+                return tmp[len(self.mediawiki.category_prefix) + 1:]
             return tmp
 
         tmp = [_get_cat(link) for link in results.get("categories", list())]
